% PURPOSE: Moran calc for VACpo ... (88 93 98 03) RCP  
%          (6 Edos/499 Muns consolidated from 2,183 Muns)...
%          to examine the spatial dependence
%          of a vector-variable as W Mtx function
%-------------------------------------------------------------------
% USAGE: Ilocal(stdvar,W)
%        nonormmoran(stdvar,const,W)
%-------------------------------------------------------------------
 
clear all;       % removes all variables, globals, functions and MEX links
close all;       % deletes all figures whose handles are not hidden
clear functions; % removes all compiled M- and MEX-functions
clc;             % clears the command window and homes the cursor

% =========================================================================
% Descripcin de las bases de datos empleadas ...
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% 1) Contenido de WMRCP_499.mat
     
     load WMRCP_499.mat

%    DMtrx - Distancia entre Capitales, metros    NO estandarizada
%    QO1   - Queen Orden 1 (1,0)                  NO estandarizada
%    DE0   - Q1 c/ Dist Euclidiana (de,0)         NO estandarizada
%    DE1   - Q1 c/ Dist Euclidiana (100/de,0)     NO estandarizada
%    DE2   - Q1 c/ Dist Euclidiana (1,000/de^2,0) NO estandarizada
%    SDE1  - Matriz Simtrica de 1/de (euclidiana)
%            [100,000/de para i <> j ]            NO estandarizada
%    SDE2  - Matriz Simtrica de 1/de^2 (euclidiana)
%            [1000,000,000/de para i <> j ]       NO estandarizada
%    --------------------------------------------------------------
%    *\  de = Distancia Euclidiana entre Centroides de Polgonos
%             Municipales, determinados con GeoDa
%    **\ Archivo documental en : "QO1_from_GeoDa.xls"
%    --------------------------------------------------------------
%    === Building Standarized Weight Hypermatrix % ===

     W21 = normw(QO1);  clear QO1;
     W22 = normw(DE1);  clear DE1;
     W23 = normw(DE2);  clear DE2;
     W24 = normw(SDE1); clear SDE1;
     W25 = normw(SDE2); clear SDE2;
  
     v2wmtx = strvcat ('QO1','DE1','DE2','SDE1','SDE2');
% -------------------------------------------------------------------------
% 2) Contenido de RCP_CE499.mat

     load RCP_CE499.mat

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%    01-ID
%    02-EDO
%    03-EDO_NAME
%    04-Z88  08-XL88  12-XLREL88
%    05-Z93  09-XL93  13-XLREL93
%    06-Z98  10-XL98  14-XLREL98  16-XCOO_GDA <- Coordenadas del centroide 	
%    07-Z03  11-XL03  15-XLREL03  17-YCOO_GDA <- poligonal c/GeoDa
% =========================================================================
% === Para ejercicio con transformacines varias de : var = var(z) ===
% OJO: Queda escrita la rutina para no-relativizacin y relativizacin...

    wx = [4,5,6,7];     wrkv = 'Z = VAC / PO';            % to work Z
  % wx = [8,9,10,11]; wrkv = 'XL = ln(Z)';                % to work XL
  % wx = [12,13,14,15]; wrkv = 'XLrel = ln(Z)/mu[ln(Z)]'; % to work XLrel

% --- --- --- --- --- --- --- --- ---  
  
    x1(:,1) = data(:,wx(1));  % define raw X[obs, ser] for 1988
    x1(:,2) = data(:,wx(2));  % define raw X[obs, ser] for 1993
    x1(:,3) = data(:,wx(3));  % define raw X[obs, ser] for 1998
    x1(:,4) = data(:,wx(4));  % define raw X[obs, ser] for 2003
  
%  x1(:,1) = data(:,wx(1))./mean(data(:,wx(1)));  % define raw X[obs, ser] for 1988
%  x1(:,2) = data(:,wx(2))./mean(data(:,wx(2)));  % define raw X[obs, ser] for 1993
%  x1(:,3) = data(:,wx(3))./mean(data(:,wx(3)));  % define raw X[obs, ser] for 1998
%  x1(:,4) = data(:,wx(4))./mean(data(:,wx(4)));  % define raw X[obs, ser] for 2003


% === Defining study variables ===

% --- Do Z [VACpo]

    Z88 = x1(:,1);   % 
    Z93 = x1(:,2);   % 
    Z98 = x1(:,3);   % 
    Z03 = x1(:,4);   % 
 
  studyvar2 = [Z88 Z93 Z98 Z03];
  vname2 = strvcat('Z88','Z93','Z98','Z03');
  clear Z88; clear Z93; clear Z98; clear Z03;

% --- Do XL [ ln(Z) ]
 
%   XL88 = x1(:,1);   % 
%   XL93 = x1(:,2);   % 
%   XL98 = x1(:,3);   % 
%   XL03 = x1(:,4);   % 
 
%   studyvar2 = [XL88 XL93 XL98 XL03];
%   vname2 = strvcat('XL88','XL93','XL98','XL03');
%   clear XL88; clear XL93; clear XL98; clear XL03;
   
% --- Do XLrel [ ln(Z) / mean(ln(Z)) ]

%   XLrel88 = x1(:,1);   % 
%   XLrel93 = x1(:,2);   % 
%   XLrel98 = x1(:,3);   % 
%   XLrel03 = x1(:,4);   % 

%   studyvar2 = [XLrel88 XLrel93 XLrel98 XLrel03];
%   vname2 = strvcat('XLrel88','XLrel93','XLrel98','XLrel03');
%   clear XLrel88; clear XLrel93; clear XLrel98; clear XLrel03;
            
% === Moran I and pvalue calculation ===
 
  % --- Doing for you ---
  
  j = 4; % Variable -> m (1:j:4)
% k = 5; % W Matrix -> n (1:k:5)
    
    for m = 1:j 
        fprintf(1,'=============  Indice de Moran  ============= %2g \n',m)
          
% -------------------

          usedwm = v2wmtx(1,:);
          usedvar = vname2(m,:);
          variable = studyvar2(:,m);
          WX=W21*variable;
          stdvar=variable/std(variable,1); 
          
             Iloc=Ilocal(stdvar,W21);
             Iloc=Iloc';
          
                const=ones(length(variable),1);
                res=nonormmoran(stdvar,const,W21);
                Imoran=[res.morani];
                pvalue=1-norm_cdf(Imoran,0,1);
          
          fprintf(1,' \n')
          fprintf(1,'-------------  Indice de Moran  ------------- %2g \n',1)
          fprintf(1,'-- Variable: %17s \n',usedvar);
          fprintf(1,'-- W Matrix: %17s \n',usedwm);
          fprintf(1,'I de Moran = %12.4f \n',Imoran); IdeMrn(m,1) = Imoran;
          fprintf(1,'   p-value = %12.4f \n',pvalue);   pVlr(m,1) = pvalue;
          fprintf(1,'      prob = %12.4f \n',res.prob); Prbt(m,1) = res.prob;
          fprintf(1,' \n');
          
% -------------------
 
          usedwm = v2wmtx(2,:);
          usedvar = vname2(m,:);
          variable = studyvar2(:,m);
          WX=W22*variable;
          stdvar=variable/std(variable,1); 
          
             Iloc=Ilocal(stdvar,W22);
             Iloc=Iloc';
          
                const=ones(length(variable),1);
                res=nonormmoran(stdvar,const,W22);
                Imoran=[res.morani];
                pvalue=1-norm_cdf(Imoran,0,1);
          
          fprintf(1,' \n')
          fprintf(1,'-------------  Indice de Moran  ------------- %2g \n',2)
          fprintf(1,'-- Variable: %17s \n',usedvar);
          fprintf(1,'-- W Matrix: %17s \n',usedwm);
          fprintf(1,'I de Moran = %12.4f \n',Imoran); IdeMrn(m,2) = Imoran;
          fprintf(1,'   p-value = %12.4f \n',pvalue);   pVlr(m,2) = pvalue;
          fprintf(1,'      prob = %12.4f \n',res.prob); Prbt(m,2) = res.prob;
          fprintf(1,' \n');
          
% -------------------
 
          usedwm = v2wmtx(3,:);
          usedvar = vname2(m,:);
          variable = studyvar2(:,m);
          WX=W23*variable;
          stdvar=variable/std(variable,1); 
          
             Iloc=Ilocal(stdvar,W23);
             Iloc=Iloc';
          
                const=ones(length(variable),1);
                res=nonormmoran(stdvar,const,W23);
                Imoran=[res.morani];
                pvalue=1-norm_cdf(Imoran,0,1);
          
          fprintf(1,' \n')
          fprintf(1,'-------------  Indice de Moran  ------------- %2g \n',3)
          fprintf(1,'-- Variable: %17s \n',usedvar);
          fprintf(1,'-- W Matrix: %17s \n',usedwm);
          fprintf(1,'I de Moran = %12.4f \n',Imoran); IdeMrn(m,3) = Imoran;
          fprintf(1,'   p-value = %12.4f \n',pvalue);   pVlr(m,3) = pvalue;
          fprintf(1,'      prob = %12.4f \n',res.prob); Prbt(m,3) = res.prob;
          fprintf(1,' \n');  
          
% -------------------
 
          usedwm = v2wmtx(4,:);
          usedvar = vname2(m,:);
          variable = studyvar2(:,m);
          WX=W24*variable;
          stdvar=variable/std(variable,1); 
          
             Iloc=Ilocal(stdvar,W24);
             Iloc=Iloc';
          
                const=ones(length(variable),1);
                res=nonormmoran(stdvar,const,W24);
                Imoran=[res.morani];
                pvalue=1-norm_cdf(Imoran,0,1);
          
          fprintf(1,' \n')
          fprintf(1,'-------------  Indice de Moran  ------------- %2g \n',4)
          fprintf(1,'-- Variable: %17s \n',usedvar);
          fprintf(1,'-- W Matrix: %17s \n',usedwm);
          fprintf(1,'I de Moran = %12.4f \n',Imoran); IdeMrn(m,4) = Imoran;
          fprintf(1,'   p-value = %12.4f \n',pvalue);   pVlr(m,4) = pvalue;
          fprintf(1,'      prob = %12.4f \n',res.prob); Prbt(m,4) = res.prob;
          fprintf(1,' \n');
          
 % -------------------
 
          usedwm = v2wmtx(5,:);
          usedvar = vname2(m,:);
          variable = studyvar2(:,m);
          WX=W25*variable;
          stdvar=variable/std(variable,1); 
          
             Iloc=Ilocal(stdvar,W25);
             Iloc=Iloc';
          
                const=ones(length(variable),1);
                res=nonormmoran(stdvar,const,W25);
                Imoran=[res.morani];
                pvalue=1-norm_cdf(Imoran,0,1);
          
          fprintf(1,' \n')
          fprintf(1,'-------------  Indice de Moran  ------------- %2g \n',5)
          fprintf(1,'-- Variable: %17s \n',usedvar);
          fprintf(1,'-- W Matrix: %17s \n',usedwm);
          fprintf(1,'I de Moran = %12.4f \n',Imoran); IdeMrn(m,5) = Imoran;
          fprintf(1,'   p-value = %12.4f \n',pvalue);   pVlr(m,5) = pvalue;
          fprintf(1,'      prob = %12.4f \n',res.prob); Prbt(m,5) = res.prob;
          fprintf(1,' \n');  
          
    end

    
  
  