% PURPOSE: arc_moranplot(17 Vars - 6 States with 499 Counties)
%          with an ArcView shape file containing 
%          499 polygons for Muns in "RCP                                                                                            " 
%------------------------------------------------------------------
% USAGE: arc_moranplot
%------------------------------------------------------------------

clear all;       % removes all variables, globals, functions and MEX links
close all;       % deletes all figures whose handles are not hidden
clear functions; % removes all compiled M- and MEX-functions
clc;             % clears the command window and homes the cursor

% =========================================================================
% Descripcin de las bases de datos empleadas ...
% 1) Contenido de WMRCP_499.mat
     
     load WMRCP_499.mat

%    DMtrx - Distancia entre Capitales, metros    NO estandarizada
%    QO1   - Queen Orden 1 (1,0)                  NO estandarizada
%    DE0   - Q1 c/ Dist Euclidiana (de,0)         NO estandarizada
%    DE1   - Q1 c/ Dist Euclidiana (100/de,0)     NO estandarizada
%    DE2   - Q1 c/ Dist Euclidiana (1,000/de^2,0) NO estandarizada
%    SDE1  - Matriz Simtrica de 1/de (euclidiana)
%            [100,000/de para i <> j ]            NO estandarizada
%    SDE2  - Matriz Simtrica de 1/de^2 (euclidiana)
%            [1000,000,000/de para i <> j ]       NO estandarizada
%    --------------------------------------------------------------
%    *\  de = Distancia Euclidiana entre Centroides de Polgonos
%             Municipales, determinados con GeoDa
%    **\ Archivo documental en : "QO1_from_GeoDa.xls"
% -------------------------------------------------------------------------

%    W1 = QO1;   W = normw(W1);   % row-normalize W function of LeSage to standarize used Weight Matrix
%    W1 = DE1;   W = normw(W1);
%    W1 = DE2;   W = normw(W1);
     W1 = SDE1;  W = normw(W1);
%    W1 = SDE2;  W = normw(W1);
% -------------------------------------------------------------------------
% 2) Contenido de RCP_CE499.mat

%    load RCP_CE499.mat

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%    01-ID
%    02-EDO
%    03-EDO_NAME
%    04-Z88  08-XL88  12-XLREL88
%    05-Z93  09-XL93  13-XLREL93
%    06-Z98  10-XL98  14-XLREL98  16-XCOO_GDA <- Coordenadas del centroide 	
%    07-Z03  11-XL03  15-XLREL03  17-YCOO_GDA <- poligonal c/GeoDa
% =========================================================================
 
filename = 'shape_files\RCP_from_2183';
results = shape_read(filename);
nobs = results.nobs;

Z88 = results.data(:,4);
Z93 = results.data(:,5);
Z98 = results.data(:,6);
Z03 = results.data(:,7);

Zrel88 = results.data(:,4)./mean(results.data(:,4));
Zrel93 = results.data(:,5)./mean(results.data(:,6));
Zrel98 = results.data(:,6)./mean(results.data(:,7));
Zrel03 = results.data(:,7)./mean(results.data(:,8));

XL88 = results.data(:,8);
XL93 = results.data(:,9);
XL98 = results.data(:,10);
XL03 = results.data(:,11);

XLrel88 = results.data(:,12);
XLrel93 = results.data(:,13);
XLrel98 = results.data(:,14);
XLrel03 = results.data(:,15);

variable = [Z88 Z93 Z98 Z03 Zrel88 Zrel93 Zrel98 Zrel03 XL88 XL93 XL98 XL03...
            XLrel88 XLrel93 XLrel98 XLrel03];

  options.vnames = strvcat('Z88','Z93','Z98','Z03','Zrel88','Zrel93',...
                           'Zrel98','Zrel03','XL88','XL93','XL98','XL03',...
                           'XLrel88','XLrel93','XLrel98','XLrel03');
  
%        options.labels     = 1 for labels, 0 = default, no labels
%        options.missing    = a vector of 0's and 1's for missing and non-missing observations
%                             0 = missing, 1 = non-missing 
%                             (produces white map polygons for missing values)
%        options.mapmenu    = 1, for a menubar, 0 = default, no menubar
%                             (useful for printing, editing the map)
%        options.legendmenu = 1, for a menubar, 0 = default, no menubar
%                             (useful for printing, editing the map)
 
  options.labels = 1;
  options.mapmenu = 1;
  options.legendmenu = 1;
  
arc_moranplot(variable,W,results,options);
