% Use moransad.m Example A for RCP w/499 obs
% -------------------------------------------------------------------------
% Demonstrates usage of moransad.m
% -------------------------------------------------------------------------

clear all;       % removes all variables, globals, functions and MEX links
close all;       % deletes all figures whose handles are not hidden
clear functions; % removes all compiled M- and MEX-functions
clc;             % clears the command window and homes the cursor

% =========================================================================
% Descripcin de las bases de datos empleadas ...
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% 1) Contenido de WMRCP_499.mat
     
     load WMRCP_499.mat

%    DMtrx - Distancia entre Capitales, metros    NO estandarizada
%    QO1   - Queen Orden 1 (1,0)                  NO estandarizada
%    DE0   - Q1 c/ Dist Euclidiana (de,0)         NO estandarizada
%    DE1   - Q1 c/ Dist Euclidiana (100/de,0)     NO estandarizada
%    DE2   - Q1 c/ Dist Euclidiana (1,000/de^2,0) NO estandarizada
%    SDE1  - Matriz Simtrica de 1/de (euclidiana)
%            [100,000/de para i <> j ]            NO estandarizada
%    SDE2  - Matriz Simtrica de 1/de^2 (euclidiana)
%            [1000,000,000/de para i <> j ]       NO estandarizada
%    --------------------------------------------------------------
%    *\  de = Distancia Euclidiana entre Centroides de Polgonos
%             Municipales, determinados con GeoDa
%    **\ Archivo documental en : "QO1_from_GeoDa.xls"
%    --------------------------------------------------------------
% 2) Contenido de RCP_CE499.mat

     load RCP_CE499.mat

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%    01-ID
%    02-EDO
%    03-EDO_NAME
%    04-Z88  08-XL88  12-XLREL88
%    05-Z93  09-XL93  13-XLREL93
%    06-Z98  10-XL98  14-XLREL98  16-XCOO_GDA <- Coordenadas del centroide 	
%    07-Z03  11-XL03  15-XLREL03  17-YCOO_GDA <- poligonal c/GeoDa
% =========================================================================
  
  y = data(:,4); wrkv = 'Z = VAC / PO > 1988'; clear data;         % to work Z88
% y = data(:,5); wrkv = 'Z = VAC / PO > 1993'; clear data;         % to work Z93
% y = data(:,6); wrkv = 'Z = VAC / PO > 1998'; clear data;         % to work Z98
% y = data(:,7); wrkv = 'Z = VAC / PO > 2003'; clear data;         % to work Z03

  nObs = length(y);                % number of observations

  X(1:nObs,1) =1;                  % exogenous variable (a constant)
            
%  B = DMtrx; wrkm = 'DMMtrx';
%  B = QO1;  wrkm = 'QO1';
%  B = DE0;  wrkm = 'DE0';
%  B = DE1;  wrkm = 'DE1';
%  B = DE2;  wrkm = 'DE2';
   B = SDE1; wrkm = 'SDE1';
%  B = SDE1; wrkm = 'SDE2';

code = 'W';                      % coding scheme
GlobalMI = 'gl';                 % global (a. local MI)
Sad = 'j';                       % saddlepoint approximation

mi  = moransad(y,X,B,code,GlobalMI,Sad); 

  fprintf(1,'================================================== \n')

  fprintf(1,'==== RCP Results using normal approximation ====== \n')
  fprintf(1,'---- Obs: 499; WM = %4s; Var: %14s \n',wrkm,wrkv)

  fprintf(1,'   Global Moran s I    = %12.4f \n',mi.mi(1,1));
  fprintf(1,'   Expected value      = %12.4f \n',mi.moments_exp(1,1));
  fprintf(1,'   Variance            = %12.4f \n',mi.moments_var(1,1));
  fprintf(1,'   Skewness            = %12.4f \n',mi.moments_skw(1,1));
  fprintf(1,'   Kurtosis            = %12.4f \n',mi.moments_kur(1,1));
  fprintf(1,'   Probability (I<=Io) = %12.4f \n',mi.prob_nv(1,1));
  fprintf(1,'   z(Io)               = %12.4f \n',mi.z_mi(1,1));

  fprintf(1,'---------------------------------------------------- \n')

  fprintf(1,'==== Results using saddlepoint approximation ==== \n')
  fprintf(1,'--- Obs: 499; WM = %4s; Var: %14s \n',wrkm,wrkv)

  fprintf(1,'    Saddlepoint                  = %12.4f \n',mi.sad(1,1));
  fprintf(1,'    r-parameter for distribution = %12.4f \n',mi.sad_r(1,1));
  fprintf(1,'    u-parameter for distribution = %12.4f \n',mi.sad_u(1,1));
  fprintf(1,'    Probability (I<=Io)          = %12.4f \n',mi.prob_sad(1,1));

  fprintf(1,'==================================================== \n')

  fprintf(1,' \n');
