% PURPOSE: Gaussian Kernel Densities - h:Fxd -
%          to examine modes for vector-variables
%----------------------------------------------------------
% USAGE   :  ksdensity()
%----------------------------------------------------------
% Implementation based on:
% S.J. Sheather and M.C. Jones. 'A reliable data-based
% bandwidth selection method for kernel density estimation'
% J. Royal Statist. Soc. B, 53:683-690, 1991
%    Author  :  Vikas.C.Raykar
%    Date    :  25 June 2005
%    Contact :  vikas@cs.umd.edu
%----------------------------------------------------------
 
clear all;       % removes all variables, globals, functions and MEX links
close all;       % deletes all figures whose handles are not hidden
clear functions; % removes all compiled M- and MEX-functions
clc;             % clears the command window and homes the cursor
 
% =========================================================================

     load RCP_CE499F.mat
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%    01-ID
%    02-EDO
%    03-EDO_NAME
%    04-Z88  08-XL88  12-XLREL88
%    05-Z93  09-XL93  13-XLREL93
%    06-Z98  10-XL98  14-XLREL98  16-XCOO_GDA <- Coordenadas del centroide 	
%    07-Z03  11-XL03  15-XLREL03  17-YCOO_GDA <- poligonal c/GeoDa
% =========================================================================
% === Para ejercicio con transformacines varias de : var = var(z) ===
% OJO: Queda escrita la rutina para no-relativizacin y relativizacin...

  % wx = [4,5,6,7];     wrkv = 'Z = VAC / PO';            % to work Z
  % wx = [8,9,10,11];   wrkv = 'XL = ln(Z)';              % to work XL
    wx = [12,13,14,15]; wrkv = 'XLrel = ln(Z)/mu[ln(Z)]'; % to work XLrel
 
% --- --- --- --- --- --- --- --- ---  

    Z1 = data(:,wx(1));  X(1,:) = Z1';
    Z2 = data(:,wx(2));  X(2,:) = Z2';
    Z3 = data(:,wx(3));  X(3,:) = Z3';
    Z4 = data(:,wx(4));  X(4,:) = Z4';
  
 %  Z1 = data(:,wx(1))./mean(data(:,wx(1)));  X(1,:) = Z1';
 %  Z2 = data(:,wx(2))./mean(data(:,wx(2)));  X(2,:) = Z2';
 %  Z3 = data(:,wx(3))./mean(data(:,wx(3)));  X(3,:) = Z3';
 %  Z4 = data(:,wx(4))./mean(data(:,wx(4)));  X(4,:) = Z4';

% --- --- --- --- --- --- --- --- ---  

disp('----------------------------------------------------');
disp(sprintf('Sheather and Jones: Fast optimal bandwidth selection'));
disp('----------------------------------------------------');

[dobs CJ] = size(data);
N = dobs;

disp('====================================================');
disp(sprintf('Estimating the AMISE optimal bandwidth-SLOW method'));
disp('----------------------------------------------------');

to=clock;    

[h_slw88]=slow_univariate_bandwidth_estimate_STEPI(N,X(1,:));
[h_slw93]=slow_univariate_bandwidth_estimate_STEPI(N,X(2,:));
[h_slw98]=slow_univariate_bandwidth_estimate_STEPI(N,X(3,:));
[h_slw03]=slow_univariate_bandwidth_estimate_STEPI(N,X(4,:));

slow_time=etime(clock,to);

disp(sprintf('h_opt88 = %f,',h_slw88));
disp(sprintf('h_opt93 = %f,',h_slw93));
disp(sprintf('h_opt98 = %f,',h_slw98));
disp(sprintf('h_opt03 = %f, Time taken = %f seconds',h_slw03,slow_time));

epsil=1e-3;

disp('====================================================');
disp(sprintf('Estimating the AMISE optimal bandwidth-Fast method with epsilon=%e',epsil));
disp('----------------------------------------------------');

to=clock;    

[h_fst88]=fast_univariate_bandwidth_estimate_STEPI(N,X(1,:),epsil);
[h_fst93]=fast_univariate_bandwidth_estimate_STEPI(N,X(2,:),epsil);
[h_fst98]=fast_univariate_bandwidth_estimate_STEPI(N,X(3,:),epsil);
[h_fst03]=fast_univariate_bandwidth_estimate_STEPI(N,X(4,:),epsil);

fast_time=etime(clock,to);

disp(sprintf('h_opt88 = %f,',h_fst88));
disp(sprintf('h_opt93 = %f,',h_fst93));
disp(sprintf('h_opt98 = %f,',h_fst98));
disp(sprintf('h_opt03 = %f, Time taken = %f seconds',h_fst03,fast_time));

rel_error_88=abs((h_slw88-h_fst88)/h_slw88)*100;
rel_error_93=abs((h_slw93-h_fst93)/h_slw93)*100;
rel_error_98=abs((h_slw98-h_fst98)/h_slw98)*100;
rel_error_03=abs((h_slw03-h_fst03)/h_slw03)*100;

speedup=slow_time/fast_time;

disp('----------------------------------------------------');
disp(sprintf('Speedup=%f Relative percentage error=%d,%d,%d,%d, %% ',speedup,rel_error_88,rel_error_93,rel_error_98,rel_error_03));
disp('----------------------------------------------------');

Y1 = min(X(1,:)):0.0001:max(X(1,:));
Y2 = min(X(2,:)):0.0001:max(X(2,:));
Y3 = min(X(3,:)):0.0001:max(X(3,:));
Y4 = min(X(4,:)):0.0001:max(X(4,:));

min_x1 = min(X(1,:)); min_x2 = min(X(2,:));
min_x3 = min(X(3,:)); min_x4 = min(X(4,:));
min_y1 = min(Y1); min_y2 = min(Y2);
min_y3 = min(Y3); min_y4 = min(Y4);

shift1 = min(min_x1, min_y1); shift2 = min(min_x2, min_y2);
shift3 = min(min_x3, min_y3); shift4 = min(min_x4, min_y4);

X_shifted1 = X(1,:) - shift1; X_shifted2 = X(2,:) - shift2;
X_shifted3 = X(3,:) - shift3; X_shifted4 = X(4,:) - shift4;
Y_shifted1 = Y1 - shift1; Y_shifted2 = Y2 - shift2;
Y_shifted3 = Y3 - shift3; Y_shifted4 = Y4 - shift4;

max_x1 = max(X_shifted1); max_x2 = max(X_shifted2);
max_x3 = max(X_shifted3); max_x4 = max(X_shifted4);
max_y1 = max(Y_shifted1); max_y2 = max(Y_shifted2);
max_y3 = max(Y_shifted3); max_y4 = max(Y_shifted4);

scale1 = 1/max(max_x1,max_y1); scale2 = 1/max(max_x2,max_y2);
scale3 = 1/max(max_x3,max_y3); scale4 = 1/max(max_x4,max_y4);

X_shifted_scaled1 = X_shifted1*scale1; X_shifted_scaled2 = X_shifted2*scale2;
X_shifted_scaled3 = X_shifted3*scale3; X_shifted_scaled4 = X_shifted1*scale4;

Y_shifted_scaled1 = Y_shifted1*scale1; Y_shifted_scaled2 = Y_shifted2*scale2;
Y_shifted_scaled3 = Y_shifted3*scale3; Y_shifted_scaled4 = Y_shifted4*scale4;

M1=length(Y1); M2=length(Y2); M3=length(Y3); M4=length(Y4); 
epsil=1e-6;
[kde1]=FastUnivariateDensityDerivative(N,M1,X_shifted_scaled1,Y_shifted_scaled1,h_fst88*scale1,0,epsil);
[kde2]=FastUnivariateDensityDerivative(N,M2,X_shifted_scaled2,Y_shifted_scaled2,h_fst93*scale2,0,epsil);
[kde3]=FastUnivariateDensityDerivative(N,M3,X_shifted_scaled3,Y_shifted_scaled3,h_fst98*scale3,0,epsil);
[kde4]=FastUnivariateDensityDerivative(N,M4,X_shifted_scaled4,Y_shifted_scaled4,h_fst03*scale4,0,epsil);

figure;
% plot(Y,actual_density,'k');
% density_number=1;
% [actual_density,X]=marron_wand_normal_mixtures(density_number,Y,N);

hold on;

plot(Y1,scale1*kde1,'-r'); plot(Y2,scale2*kde2,'-b');
plot(Y3,scale3*kde3,'-g'); plot(Y4,scale4*kde4,'-m');

% plot(X(1,:),0,'k+');
legend('88','93','98','03');
