% PURPOSE: Bootstrap - Sample with replacement
%          based on Statistics Hdbk Mtz & Mtz
%          Author: J Manuel Castaeda A - 14/11/2010
%--------------------------------------------------------------------------
% USAGE:   The general statistical model is represented by the parameter M,
%          M = { h, S, alpha1, ... , alphaS, c }
% =========================================================================

clear all;       % removes all variables, globals, functions and MEX links
close all;       % deletes all figures whose handles are not hidden
clear functions; % removes all compiled M- and MEX-functions
clc;             % clears the command window and homes the cursor

% -------------------------------------------------------------------------
 
  load RCP_CE499F.mat

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%    01-ID
%    02-EDO
%    03-EDO_NAME
%    04-Z88  08-XL88  12-XLREL88
%    05-Z93  09-XL93  13-XLREL93
%    06-Z98  10-XL98  14-XLREL98  16-XCOO_GDA <- Coordenadas del centroide 	
%    07-Z03  11-XL03  15-XLREL03  17-YCOO_GDA <- poligonal c/GeoDa
% =========================================================================

% === Para ejercicio con transformacines varias de : var = var(z) ===

  % wx = [4,5,6,7];     wrkv = 'Zf = ZF(Z)';                 % to work Z
  % wx = [8,9,10,11];   wrkv = 'XLf = ln(Zf)';               % to work XL
    wx = [12,13,14,15]; wrkv = 'XLrelf = ln(Zf)/mu[ln(Zf)]'; % to work XLrel

% --- --- --- --- --- --- --- --- ---  

% INPUTS ...

  B = 1000;          % define REPLICATES number for bootstrap  
  alpha30 = (1/3);   % probability for the Sth state less and/or egual 1/3.
  alpha20 = (1/5);                
  cvg = 0.8;         % coverage probability for the confidence intervals

% Working with "00_Raw_2183a.mat up-dated on 101024":

% --- --- --- --- --- --- --- --- ---  
  
  X(:,1) = data(:,wx(1)); X(:,2) = data(:,wx(2)); % defining raw X[obs, ser]
  X(:,3) = data(:,wx(3)); X(:,4) = data(:,wx(4)); % defining raw X[obs, ser]
 
  % clear data; clear textdata; % On Hold just as needed to save RAM
  %                               take care with line 334: Col(1) for DMtrx
  
  [n m] = size(X);            % obtaining DIMENSION of transformed variable 
  
% =========================================================================
% Step 0) Optimal bandwidth determination (ksdensity procedure):
%         Ref: Bowman, A. W., and A. Azzalini, Applied Smoothing Techniques
%              for Data Analysis, Oxford University Press, 1997.
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% Title of output 1...
  disp(sprintf('VARIABLE DE TRABAJO :   %s,',wrkv));
  disp('==========================================================');
  disp(sprintf('    --- Gaussian Kernel Densities, h = h(Opt bw) ---'));
  disp(sprintf('     --- --- --- M e t h o d :   MATLAB --- --- ---'));
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  kdenam1 = '1988: KDE of X'; kdenam2 = '1993: KDE of X'; % defining title for KDE graph
  kdenam3 = '1998: KDE of X'; kdenam4 = '2003: KDE of X'; % defining title for KDE graph
  
  lblkde(1,:) = kdenam1; lblkde(2,:) = kdenam2; % needed previous to optimize
  lblkde(3,:) = kdenam3; lblkde(4,:) = kdenam4; % needed previous to optimize
  
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  
  % Loop to built KDE graphs of selected variable or transformation
  % Expected output: f(x) on kboot[obs, ser),
  %                  x on kx[obs, ser],
  %                  u(opt bw) on ku[obs, ser] to be checked and deleted.
    for i=1:m
      % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      XJ = X(:,i);             % defining var/transf to KDE
      [f,x,u] = ksdensity(XJ); % calculating KDE
        kboot(:,i) = f';       % making f(xi) column vector - i = serie
        kx(:,i) = x';          % making xi column vector    - i = serie
        ku(i,:) = u;           % building column vector of opt bw
      % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      % Doing KDE graph output of raw var/transf...
        figure(1)
        subplot(2,2,i)
        fprintf(1,'Matlab Opt bw = %8.4f \n',ku(i,:));
        plot(kx(:,i),kboot(:,i),'-r');
        title(lblkde(i,:),'FontSize',10)
      % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    end

  clear XJ; clear f; clear x; clear u; clear i;          % just to save RAM

% =========================================================================
% Step 1) Draw B bstrp samples Y* w/size n from X by sampling & replacement
%         Ref: Martinez, W. L. and Martinez, A. R. (2002) "Computational
%              Statistics Handbook with MATLAB". Chapman & Hall/CRC.
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Title of output 2...
  disp('==========================================================');
  disp(sprintf('       --- Bootstrap - Sample with replacement: ---'));
  disp(sprintf('       --- --- --- M e t h o d :  O N E --- --- ---'));
  disp('----------------------------------------------------------');
  disp(sprintf('N de Obs = %10.0f,           N de Series = %8.0f.',n,m));
  disp(sprintf('N de Replicas = %5.0f.',B));
  disp('----------------------------------------------------------');
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

varnam1 = '1988: Var Y(X) of draws';    % defining title for Bar(VAR) graph
varnam2 = '1993: Var Y(X) of draws';    % - - - - - -     "     - - - - - -
varnam3 = '1998: Var Y(X) of draws';    % - - - - - -     "     - - - - - -
varnam4 = '2003: Var Y(X) of draws';    % - - - - - -     "     - - - - - -
 
lbl(1,:) = varnam1; lbl(2,:) = varnam2;       % needed previous to optimize
lbl(3,:) = varnam3; lbl(4,:) = varnam4;       % needed previous to optimize

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  % Loop to built bootstrap replicates

for j=1:B
  
for i=1:m
  % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
  % Bootstrap: resampling with replacement
  
  XI = X(:,i);
  s2(i) = var(XI,0);                 % Stat parameter to shown on Bar graph
  inds = unidrnd(n,n,B);             % Approach to save machine time
  yboot(:,:,i) = XI(inds);           % Known as Y* by Bianchi from Efron
  s2boot(i,:) = var(yboot(:,:,i),0); % Stat parameter to shown on Bar graph
  seb(i) = std(s2boot(i,:),0);       % Just to make sense w/booststrap qlty
  
  % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  % Doing Bar graph output of bootstrapped var/transf...
  % ... to avoid show of draws, just change to next step between ends...
  
  % hold on
      figure(2)
      subplot(2,2,i)
      hist (s2boot(i,:),20);
      title(lbl(i,:),'FontSize',10)
      gph = findobj(gca,'Type','patch');
      set(gph,'FaceColor',[0.5,0.9,1],'EdgeColor','b')
  % hold off
  
  % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
end

  % Doing Bar graph output of bootstrapped var/transf...
  % ... to avoid show of draws, just change to next step between ends...
  %figure(2)
  %subplot(2,2,i)
  %hist (s2boot(i,:),20);
  %title(lbl(i,:),'FontSize',10)
  %gph = findobj(gca,'Type','patch');
  %set(gph,'FaceColor',[0.5,0.9,1],'EdgeColor','b')

end

clear XI; clear i; clear j; clear inds;

% =========================================================================
% Step 2) Define rescaled bstrp samples X* as in Efron and Tibshirani.
%         Ref: Efron, B. and Tibshirani, R. (1993). "An Introduction to the
%              Bootstrap". Chapman and Hall. New York.
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%>disp('');
%>  disp('THETA es el estadistico bstrap (VARIANZA) para las series:');
%>  disp('                ------------------------------------------');
%>  disp('                     1988       1993       1998       2003');
%>  disp('----------------------------------------------------------');
%>  disp(sprintf('   THETA(X)  = %10.6f,%10.6f,%10.6f,%10.6f',s2(1),s2(2),s2(3),s2(4)));
%>  disp(sprintf('SD[THETA(B)] = %10.6f,%10.6f,%10.6f,%10.6f',seb(1),seb(2),seb(3),seb(4)));
%>  disp('==========================================================');
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  % Loop to built X* = xe and e(i) to be checked w/ RBL ... ON HOLD ...

    for i=1:m
        
        % ybs = yboot(:,:,i); % On hold to verify optimum time running
        ym(i,:) = mean(yboot(:,:,i),1);
        
    for j=1:B
        
        % La siguiente lnea demanda el sustento estadstico que permita
        % justificar la interpretacin operativa que se hace de Bianchi...
        err(:,j,i) = X(:,i) - yboot(:,j,i);
        
        % Importante: Ec 5 de Bianchi (1999) vs Efron (1993) para el factor
        %             de escala: (ku(i)^2)*(s2(i)^2) vs (ku(i)^2)/(s2(i)^2)
        xe(:,j,i) = ym(i,j)+((1+(ku(i)^2)/(s2(i)^2))^-0.5)*(yboot(:,j,i)-ym(i,j)+ku(i)*err(:,j,i));
      
    end
    
        % Take care : This loop needs to be validated in depth... Based on
        %             two notes written up ...
    
    end

   clear i; clear j;
%  clear ym; clear err; 

% =========================================================================
% Step 3) For each boostrap sample x*, estimate the density fh(x*), and
%         given alpha, obtain de critical value pairs...
%         Ref: Bianchi, M. and Zoega, G. (1999) "A Nonparametric Analysis
%              of Regional Unemployment Dynamics in Britain". Journal of
%              Business & Economic Statistics, Vol. 17, No. 2 (Apr., 1999),
%              pp. 205-216. (See procedure on page 207).
%         NOTE 'on' used symbology: x* has been done & called xe(n,B,yr)...
%              n = obs(2,183), B = replicates(1,000) & yr = Series(4).
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Step 3.a) Obtaining (fh(x*),x*,u*) 
%           1000(B:j) por cada una de las cuatro(m:i) series ...

    for i=1:m

        XFJ = xe(:,:,i);                           % OJO : Timing trade-off
        
    for j=1:B
        
        [fxi,xfi,u] = ksdensity(XFJ(:,j),'width',ku(i));     % oK - Ckecked
        FXe(:,j,i) = fxi; XFe(:,j,i) = xfi; uFe(j,i) = u;

    end

        clear XFJ; clear XFJj; clear fxi; clear xfi; clear u;

    end

    clear XFJ; clear XFJj; clear fxi; clear xfi; clear u; clear i; clear j;

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Step 3.b) Obtaining Critical Value Pairs (alpha is given) -> (cvx,cvf):
%           {c1*(b),c2*(b)}para toda b=1->B.

  [oxf rxf sxf] = size(XFe);        % Dim X*: o=Obs[KDE], r=Reps & s=Series

% Highest loop to work with considered series

for k=1:sxf
    
    xKDE = XFe(:,:,k);
    fKDE = FXe(:,:,k);
    
% Medium loop looking for replicates to be done on defined 'B'

    for j=1:rxf

        % Low loop to calculate areas under KDE: 'i' is related to Obs on Rows
        
        for i=1:oxf-1
            xfxtmp(i+1,1) = (xKDE(i+1,j)-xKDE(i,j))*(fKDE(i+1,j)+fKDE(i,j))/2; % Area(i)
        end

        % Low loop to calculate normalized area under KDE
        
        for i=1:oxf-1
            xfxtmp(i+1,2) = xfxtmp(i+1,1)/sum(xfxtmp(:,1)); % Normalizaed area(i)
        end

        % Low loop to calculate accumulated normalizaed area under KDE
          xfxtmp(1,3) = 0; % Required to define 'xfxa' increasing in size
            
        for i=1:oxf-1
            xfxtmp(i+1,3) = xfxtmp(i+1,2)+ xfxtmp(i,3); % Accumulated area(i)
        end

        xfxa(:,j,k) = xfxtmp(:,3); %clear xfxtmp;
        
        % Building up c1 & c2 from area's matrix based on [100, 1000, 4]

          alph1 = alpha30; alph2 = 1 - alpha30;  % OJO: Cambio ck por alphk

          xBc(j,1,k) = max(find(xfxa(:,j,k)<=alph1)); % Col 1 = Bc11(rxf) 
          xBc(j,2,k) = XFe(xBc(j,1,k),j);             % Col 2 = xC11(rxf) ... Vector to do C11LO & C11HI
          xBc(j,3,k) = FXe(xBc(j,1,k),j);             % Col 3 = FC11(rxf)

          xBc(j,4,k) = min(find(xfxa(:,j,k)>=alph2)); % Col 4 = Bc12(rxf) 
          xBc(j,5,k) = XFe(xBc(j,4,k),j);             % Col 5 = xC12(rxf) ... Vector to do C12LO & C12HI
          xBc(j,6,k) = FXe(xBc(j,4,k),j);             % Col 6 = FC12(rxf)
          
          xBc(j,7,k) = j;
          
          alph21 = alpha20; alph22 = alph21 + alpha20;
          alph23 = alph22 + alpha20; alph24 = alph23 + alpha20;
          
          xBc(j,8,k) = max(find(xfxa(:,j,k)<=alph21)); % Col 1 = Bc21(rxf) 
          xBc(j,9,k) = XFe(xBc(j,8,k),j);              % Col 2 = xC21(rxf) ... Vector to do C21LO & C21HI
          xBc(j,10,k) = FXe(xBc(j,8,k),j);             % Col 3 = FC21(rxf)
          
          xBc(j,11,k) = max(find(xfxa(:,j,k)<=alph22)); % Col 1 = Bc22(rxf) 
          xBc(j,12,k) = XFe(xBc(j,11,k),j);             % Col 2 = xC22(rxf) ... Vector to do C23LO & C23HI
          xBc(j,13,k) = FXe(xBc(j,11,k),j);             % Col 3 = FC22(rxf)
          
          xBc(j,14,k) = max(find(xfxa(:,j,k)<=alph23)); % Col 1 = Bc23(rxf) 
          xBc(j,15,k) = XFe(xBc(j,14,k),j);             % Col 2 = xC23(rxf) ... Vector to do C23LO & C23HI
          xBc(j,16,k) = FXe(xBc(j,14,k),j);             % Col 3 = FC23(rxf)
          
          xBc(j,17,k) = max(find(xfxa(:,j,k)<=alph24)); % Col 1 = Bc24(rxf) 
          xBc(j,18,k) = XFe(xBc(j,17,k),j);             % Col 2 = xC24(rxf) ... Vector to do C24LO & C24HI
          xBc(j,19,k) = FXe(xBc(j,17,k),j);             % Col 3 = FC24(rxf)

    end
    
end



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Step 4) Derive the confidence limits ckb cka by taking the {(B-Bc)}th
%         and the {Bc}th largest of the B replicates cki, i=1,2.
%         Ref: Bianchi, M. and Zoega, G. (1999)
%
%         Method of bootstrap percentile interval to achieve ck[Lo, Hi]
%         by coverage performance WITH meaning range-preserving

  BcLow = floor((1-cvg)*B)+1;  BcHigh = floor(cvg*B)+1;
  
for k = 1:sxf 
  
  CMtrx(2,k) = median(xBc(:,2,k));                         % Calcula c11 para la serie k
  tmpc1 = xBc(:,2,k); stmpc1 = sort(tmpc1);
  CMtrx(1,k) = stmpc1(BcLow); CMtrx(3,k) = stmpc1(BcHigh); % Calcula c11(low) & c11(high)
  
  CMtrx(5,k) = median(xBc(:,5,k));                         % Calcula c12 para la serie k
  tmpc2 = xBc(:,5,k); stmpc2 = sort(tmpc2);
  CMtrx(4,k) = stmpc2(BcLow); CMtrx(6,k) = stmpc2(BcHigh); % Calcula c12(low) & c12(high)
  
  clear tmpc1; clear tmpc2; clear stmpc1; clear stmpc2;
  
  CMtrx(8,k) = median(xBc(:,9,k));                             % Calcula c21 para la serie k
  tmpc21 = xBc(:,9,k); stmpc21 = sort(tmpc21);
  CMtrx(7,k) = stmpc21(BcLow); CMtrx(9,k) = stmpc21(BcHigh);   % Calcula c21(low) & c21(high)
  
  CMtrx(11,k) = median(xBc(:,12,k));                           % Calcula c22 para la serie k
  tmpc22 = xBc(:,12,k); stmpc22 = sort(tmpc22);
  CMtrx(10,k) = stmpc22(BcLow); CMtrx(12,k) = stmpc22(BcHigh); % Calcula c22(low) & c22(high)  
  
  CMtrx(14,k) = median(xBc(:,15,k));                           % Calcula c23 para la serie k
  tmpc23 = xBc(:,15,k); stmpc23 = sort(tmpc23); 
  CMtrx(13,k) = stmpc23(BcLow); CMtrx(15,k) = stmpc23(BcHigh); % Calcula c23(low) & c23(high)
  
  CMtrx(17,k) = median(xBc(:,18,k));                           % Calcula c24 para la serie k
  tmpc24 = xBc(:,18,k); stmpc24 = sort(tmpc24);
  CMtrx(16,k) = stmpc24(BcLow); CMtrx(18,k) = stmpc24(BcHigh); % Calcula c24(low) & c24(high) 
  
  clear tmpc21; clear tmpc22; clear tmpc23; clear tmpc24;
  clear stmpc21; clear stmpc22; clear stmpc23; clear stmpc24;
  
end
  

for i = 1:n
 
    DMtrx(i,1) = data(i,1); DMtrx(i,2) = X(i,1);                           % OJO ...
 
    if X(i,1) <= CMtrx(2,1); DMtrx(i,1+5) = 0; end                         % S10 para serie inicial
    if (X(i,1) > CMtrx(2,1) && X(i,1) < CMtrx(5,1)); DMtrx(i,1+5) = 1; end % S11 para serie inicial
    if (X(i,1) >= CMtrx(5,1));  DMtrx(i,1+5) = 2; end                      % S12 para serie inicial
    
    if X(i,1) <= CMtrx(8,1); DMtrx(i,9+1) = 0; end                            % S20 para serie inicial
    if (X(i,1) > CMtrx(8,1) && X(i,1) <= CMtrx(11,1)); DMtrx(i,9+1) = 1; end  % S21 para serie inicial
    if (X(i,1) > CMtrx(11,1) && X(i,1) <= CMtrx(14,1)); DMtrx(i,9+1) = 2; end % S23 para serie inicial
    if (X(i,1) > CMtrx(14,1) && X(i,1) <= CMtrx(17,1)); DMtrx(i,9+1) = 3; end % S24 para serie inicial
    if (X(i,1) > CMtrx(17,1));  DMtrx(i,9+1) = 4; end                         % S25 para serie inicial
 
end
    
for j = 2:m
 
    DMtrx(:,j+1) = X(:,j);
 
    for i = 1:n
        
        if (X(i,j) <= CMtrx(1,j)); DMtrx(i,j+5) = 0; end                         % So para T = t + 1
        if (X(i,j) > CMtrx(1,j) && X(i,j) < CMtrx(3,j)); DMtrx(i,j+5) = 5; end   % 1ra zona de indeterminancia
        if (X(i,j) >= CMtrx(3,j) && X(i,j) <= CMtrx(4,j)); DMtrx(i,j+5) = 1; end % S1 para T = t + 1
        if (X(i,j) > CMtrx(4,j) && X(i,j) < CMtrx(6,j)); DMtrx(i,j+5) = 6; end   % 2da zona de indeterminancia
        if (X(i,j) >= CMtrx(6,j));  DMtrx(i,j+5) = 2; end                        % S2 para T = t + 1
 
        if (X(i,j) <= CMtrx(8,j)); DMtrx(i,j+9) = 0; end                           % So para T = t + 1
        if (X(i,j) > CMtrx(8,j) && X(i,j) < CMtrx(11,j)); DMtrx(i,j+9) = 1; end    % S1 para T = t + 1
        if (X(i,j) >= CMtrx(11,j) && X(i,j) <= CMtrx(14,j)); DMtrx(i,j+9) = 2; end % S2 para T = t + 1
        if (X(i,j) > CMtrx(14,j) && X(i,j) < CMtrx(17,j)); DMtrx(i,j+9) = 3; end   % S3 para T = t + 1
        if (X(i,j) >= CMtrx(17,j));  DMtrx(i,j+9) = 4; end                         % S4 para T = t + 1
        
    end
 
end

