% PURPOSE: Distribution Dynamics Analysis
%          Working Paper - Distribution Dynamics
%          Code for analysis reported in FIGURES 4, 5, 6 and 7
%          Author: Magrini, S. - 27/08/2007
%-------------------------------------------------------------
% USAGE: Distribution_CE()        ... adapted by JMCA - 101220
%-------------------------------------------------------------
 
clear all;       % removes all variables, globals, functions and MEX links
close all;       % deletes all figures whose handles are not hidden
clear functions; % removes all compiled M- and MEX-functions
clc;             % clears the command window and homes the cursor

% =========================================================================

     load RCP_CE499Joint.mat
     load RCP_Escol.mat
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%    01-ID
%    02-EDO
%    03-EDO_NAME
%    04-Z88  08-XL88  12-XLREL88
%    05-Z93  09-XL93  13-XLREL93
%    06-Z98  10-XL98  14-XLREL98  16-XCOO_GDA <- Coordenadas del centroide 	
%    07-Z03  11-XL03  15-XLREL03  17-YCOO_GDA <- poligonal c/GeoDa
% =========================================================================
% === Para ejercicio con transformacines varias de : var = var(z) ===
% OJO: Queda escrita la rutina para no-relativizacin y relativizacin...

  % wx = [4,5,6,7];     wrkv = 'Zf = ZF(Z)';        alpha = --- ;
  % wx = [8,9,10,11];   wrkv = 'XLf = ln(Zf)';      alpha = --- ;
    wx = [12,13,14,15]; wrkv = 'XLrelF(03) & Escol(00)'; alpha = 0.1041;

         % yr 1988:  h(XLrel) = 0.1058, h(XLrelF) = 0.0982 -> 0.1020
         % yr 2003:  h(XLrel) = 0.1211, h(XLrelF) = 0.1113 -> 0.1162
         % yr 2000:  h(Escol) = 0.0918,                      
                    
% --- --- --- --- --- --- --- --- ---    
  
% varRaw = data_raw(:,wx(4));  % define raw X[obs, ser] for XLrel
  varFil = data_fil(:,wx(4));  % define raw X[obs, ser] for XLrelF
  varEsc = data_esc(:,4);      % conditioning escol

% --- --- --- --- --- --- --- --- ---

% x0 = varRaw; x1 = varFil; % Periodo analizado
  x0 = varFil; x1 = varEsc; % Periodo analizado
  
  % x0r = x0; x1r = x1;
  
  x0r = x0./mean(x0);   % Para la variable relativizada : var / mu
  x1r = x1./mean(x1);   % Para la variable relativizada : var / mu
    
% XYdata=[x0r  x1r]; Xdataname='XLrel03'; Ydataname='XLrelF03';
  XYdata=[x0r  x1r]; Xdataname='XLrelf03'; Ydataname='Escol00';
 
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
%%%     Other Inputs
bw = 'h2_dpi';              % Bandwidth
int=50;                     % Number of non-overlapping intervals
k='gauss';                  % Kernel
% alpha = 0.5;              % Sensitivity parameter for adaptive kernel (0 for fixed bandwidths)
iterations = 200;           % Number of iterations for trasitional dynamics (0 for half-life)
% levels = [95 85 75 65 55 45 35 25 15 5];       % Percentage contour levels: 90 75 50 25 10
  levels = [96 86 82 76 66 54 46 34 22 10 6 1];  % Percentage contour levels: 90 75 50 25 10
savefigures = 0;            % Save figures (0 = NO; 1 = YES)
 
%%%%    ANALYSIS
[z,P,d_joint,d_marginalN,d_iter,half_life,iterg,d_ergodic]=ske(XYdata,bw,int,k,iterations,alpha);
half_life;                  % estimates the half-life of the chain

  disp(sprintf('VARIABLE DE TRABAJO :   %s,',wrkv));
  disp('==========================================================');
  fprintf(1,' --- Dinmica de la Distribucin --- \n');
  fprintf('       Perodo:    %s-%s \n',Xdataname,Ydataname);
  fprintf(1,'        alpha:     %8.4f \n',alpha);
  fprintf(1,'        Half Life: %8.4f \n',half_life);
  disp('==========================================================');
 
%%%%    GRAPHIC OUTPUT
ssize=z(2)-z(1); 
rP=(P/(sum(sum(P))*ssize*ssize));  % re-scale P (so that volume of P equals 1)
 
%   1   plot of initial vs. final distribution
figure('Position',[100 100 450 400]);
d_finalN=d_marginalN*P;
observed=plot(z,d_marginalN,'k','LineWidth',2);
hold on; plot(z,d_finalN,'k --','LineWidth',2); hold off;
legend(Xdataname,Ydataname,0); 
legend(strcat(Xdataname),strcat(Ydataname));
 
%   2   3D surface plot of the stochastic kernel
figure('Position',[200 120 450 400]);
sk3d=surf(z,z,rP,'EdgeColor','k');
colormap(white);
view([-12 58]); axis tight; xlabel(Ydataname); ylabel(Xdataname);
 
%   3   percentage contour plot of the stochastic kernel
skplot=figure('Position',[300 140 450 400]);
pvl=pvlevs(rP,levels);
[C,h]=contour(z,z,rP,pvl); xlabel(Ydataname); ylabel(Xdataname); 
Clev=rncl(C,h,levels); 
set(findobj('Type','patch'),'LineWidth',1,'EdgeColor','k'); clabel(Clev,h);
hold on; a1 = min(z)*0.6:.05:max(z)*1.4; a2 = a1; plot(a1,a2,'--k','LineWidth',2); hold off;
 
%   4   plot of initial distribution vs. distribution after some iterations
figure('Position',[400 160 450 400]);
if  iterations == 0,
    iterations = round(half_life);
end
trans=plot(z,d_marginalN,'k','LineWidth',2);
hold on; plot(z,d_iter,'k --','LineWidth',2); hold off;
legend(Xdataname, ['after ', num2str(iterations), ' iterations']);
 
%   5   plot of initial vs. ergodic distribution
figure('Position',[500 180 450 400]);
ergodic=plot(z,d_marginalN,'k','LineWidth',2);
hold on; plot(z,d_ergodic,'k --','LineWidth',2); hold off;
legend(Xdataname,'ergodic',0); 
legend(strcat(Xdataname), 'ergodic');
 
%   6   check whether to save figures or not
if  savefigures == 1
    saveas(observed,'observed');
    saveas(sk3d,'sksurf');
    saveas(skplot,'skcont');
    saveas(trans,'trans');
    saveas(ergodic,'ergodic');
end
