% Use objfct.m Ln(Z)rel for RCP w/499 obs
% -------------------------------------------------------------------------
% finds optimal delta for objfct.m, optimal weights matrix and
% filtered variable
% -------------------------------------------------------------------------
% OUTPUT:
% a structure variable
%            gi.g = G_i statistic
%          gi.e_g = Erwartungswert
%           gi.y1 = Y1-parameter for variance
%           gi.y1 = Y2-parameter for variance
%          gi.s_g = standard deviation
%          gi.z_g = z-standardized G_i
%          gi.x_f = filtered Variable
%         gi.z_mi = z-standardized Moran's I 
%                   (of filtered variable)
%
%               w = row-sum standardized weights matrix
%               u = unstandardized weights matrix
% 
%           delta = optimal delta
% -------------------------------------------------------------------------

clear all;       % removes all variables, globals, functions and MEX links
close all;       % deletes all figures whose handles are not hidden
clear functions; % removes all compiled M- and MEX-functions
clc;             % clears the command window and homes the cursor

% =========================================================================
% Descripcin de las bases de datos empleadas ...
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% 1) Contenido de WMRCP_499.mat
     
     load WMRCP_499.mat

%    DMtrx - Distancia entre Capitales, metros    NO estandarizada
%    QO1   - Queen Orden 1 (1,0)                  NO estandarizada
%    DE0   - Q1 c/ Dist Euclidiana (de,0)         NO estandarizada
%    DE1   - Q1 c/ Dist Euclidiana (100/de,0)     NO estandarizada
%    DE2   - Q1 c/ Dist Euclidiana (1,000/de^2,0) NO estandarizada
%    SDE1  - Matriz Simtrica de 1/de (euclidiana)
%            [100,000/de para i <> j ]            NO estandarizada
%    SDE2  - Matriz Simtrica de 1/de^2 (euclidiana)
%            [1000,000,000/de para i <> j ]       NO estandarizada
%    --------------------------------------------------------------
%    *\  de = Distancia Euclidiana entre Centroides de Polgonos
%             Municipales, determinados con GeoDa
%    **\ Archivo documental en : "QO1_from_GeoDa.xls"
%    --------------------------------------------------------------
% 2) Contenido de RCP_CE499.mat

     load RCP_CE499.mat

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%    01-ID
%    02-EDO
%    03-EDO_NAME
%    04-Z88  08-XL88  12-XLREL88
%    05-Z93  09-XL93  13-XLREL93
%    06-Z98  10-XL98  14-XLREL98  16-XCOO_GDA <- Coordenadas del centroide 	
%    07-Z03  11-XL03  15-XLREL03  17-YCOO_GDA <- poligonal c/GeoDa
% =========================================================================

% x = data(:,12); wrkv = 'Ln(Z)rel > 1988'; clear data; % to work XLREL88
% x = data(:,13); wrkv = 'Ln(Z)rel > 1993'; clear data; % to work XLREL93
% x = data(:,14); wrkv = 'Ln(Z)rel > 1998'; clear data; % to work XLREL98
  x = data(:,15); wrkv = 'Ln(Z)rel > 2003'; clear data; % to work XLREL03

  d = DMtrx; wrkm = 'DMtrx';
% d = QO1;  wrkm = 'QO1';
% d = DE0;  wrkm = 'DE0';
% d = DE1;  wrkm = 'DE1';
% d = DE2;  wrkm = 'DE2';
% d = SDE1; wrkm = 'SDE1';
% d = SDE2; wrkm = 'SDE2';
% =========================================================================
% Minimize objective function:
% ---------------------------------------------------------------------

[delta] = fminbnd(@RcpObjFctNew,1,200);  % 200 =< que distancia critica
                                      % buscar variando de alto a bajo
                                      % hasta encontrar el d < d(parmetro)

% ---------------------------------------------------------------------
% Optimal weights matrix:
% ---------------------------------------------------------------------

  [w,u] = distance2weight(d,delta);   % convert to
                                      % weights matrix
% ---------------------------------------------------------------------
% Optimal filtered variable:
% ---------------------------------------------------------------------

gi = getis(w,x);                              % filter variable

  fprintf(1,'================================================ \n')

  fprintf(1,'== Results: [w,u] = distance2weight(d,delta); == \n')
  fprintf(1,'--- Obs: 499; WM = %4s; Var: %14s \n',wrkm,wrkv)
  fprintf(1,' \n')
  fprintf(1,'    Delta = %12.4f \n',delta);  
  fprintf(1,' \n')

  fprintf(1,'================================================ \n')
